@@ -27,9 +27,9 @@ class FrequencySweepWorker(nsf_thread.NSFBackgroundWorker):
     """ parameter for the background work"""
     par_cantilever = None
     par_excitation_method : sweep_settings.ExcitationMethodID = sweep_settings.ExcitationMethodID.PiezoDrive 
-    par_input_source = freq_sweep.InputSource.Deflection
-    par_output_source = freq_sweep.FrequencySweepOutput.Normal_Excitation
-    par_bandwidth = freq_sweep.Bandwidths.Hz_360
+    par_input_source:freq_sweep.InputSource = freq_sweep.InputSource.Deflection
+    par_output_source:freq_sweep.FrequencySweepOutput = freq_sweep.FrequencySweepOutput.Normal_Excitation
+    par_bandwidth:freq_sweep.Bandwidths = freq_sweep.Bandwidths.Hz_360
 
     par_center_frequency = 150000
     par_frequency_range = 100000
