@@ -65,6 +65,7 @@ class SpecWorker(nsf_thread.NSFBackgroundWorker):
             lockin = self.spm.lowlevel.ctrlunits.get_lock_in()
             dacout = self.spm.lowlevel.ctrlunits.get_dac(OutputChannel_ID_to_LUInst_Map[self.par_output_id])
             original_out = dacout.dc
+            original_source = dacout._lu.input.value
 
             self.cur_voltage_range = self.par_output_span
             self.cur_steps = self.par_steps 
@@ -103,6 +104,7 @@ class SpecWorker(nsf_thread.NSFBackgroundWorker):
 
             # restore original setting of output        
             dacout.dc = original_out
+            dacout._lu.input.value = original_source
         self._disconnect_from_controller()
 
     def get_result(self) -> SpecWorkerData:
