@@ -5,16 +5,16 @@ License - MIT
 import time
 import math
 from PySide2.QtCore import Signal
-from nanosurf.lib.datatypes import nsf_thread
+import nanosurf as nsf
 from app import app_common, module_base
 from modules.demo_module import settings
 
 class MyWorkerData():
     def __init__(self):
-        self.value = []
+        self.value:list[float] = []
         self.last_index = -1
 
-class MyWorker(nsf_thread.NSFBackgroundWorker):
+class MyWorker(nsf.nsf_thread.NSFBackgroundWorker):
     """ This class implements the long lasting activity in the background to not freeze the gui """
 
     sig_tick = Signal() # is emitted if par_send_tick is True
