@@ -293,7 +293,6 @@ class StudioScriptContext(StudioScriptInterface):
         return ok
 
     def load_studio_root_table(self, lua_str: str) -> Any:
-        res_ok = False
         res_val: dict = None
         return_val = self.execute_command(f"return {lua_str}")
         if return_val is not None:
@@ -306,10 +305,6 @@ class StudioScriptContext(StudioScriptInterface):
                 if not lua_ok:
                     self._last_error = f"Scripting: {str(res_val)}"
                     res_val = None
-                
-                # re = self._lua_math_type_check_func("test.num_f")
-
-                res_ok = True
             except lupa._lupa.LuaError as er:
                 self._last_error = f"Lupa Error: {er}:\n{return_val}"
             except lupa._lupa.LuaSyntaxError:
@@ -391,6 +386,7 @@ class StudioScriptContext(StudioScriptInterface):
         return ok
 
     def _init_cmd_tree(self) -> bool:
+        print(f"Init context: {self._context_id.name.lower()}")
         compiler = studio_wrapper.CmdTreeCompiler(self)
         root_table = self.load_studio_root_table("root")
         ok = compiler.build_wrapper_class(self._context_id.name.lower(), root_table)
