@@ -1,6 +1,5 @@
 # studio node base class.py
 
-from ast import If
 import os
 import pathlib
 import enum
@@ -9,12 +8,15 @@ import lupa
 from typing import Any
 import hashlib
 import importlib
+import keyword
 import nanosurf.lib.datatypes.sci_val as sci_val
 
-g_this_compiler_version = "1.0"
+g_this_compiler_version = "2.0"
 g_context_package_name_prefix = "cmd_tree_"
 g_cmd_tree_root_name = "root"
 
+g_skip_sequencer_definition = True
+
 class LuaType(enum.Enum):
     Nil = "nil"
     Table = "list"
@@ -23,6 +25,7 @@ class LuaType(enum.Enum):
     Str = "str"
     Bool = "bool"
     Function = "function"
+    EnumType = "EnumType"
 
 def get_lua_type(obj) -> LuaType:
     """ Get the type of a lua object as LuaType enum """
@@ -42,6 +45,8 @@ def get_lua_type(obj) -> LuaType:
         return LuaType.Int
     elif lua_type == bool:
         return LuaType.Bool
+    elif isinstance(lua_type, enum.Enum):
+        return LuaType.EnumType
     return LuaType.Nil
 
 def get_lua_type_str(obj) -> str:
@@ -145,9 +150,29 @@ class CmdTreeCompiler():
                 return True
         return False
 
+    def _is_lu_trigger_func(sel, tree_name:str) -> bool:
+        try: 
+            prefix = tree_name[:7]
+            suffix = tree_name[-8:]
+            return prefix == "root.lu" and suffix == ".trigger"
+        except:
+            return False
+
+    def _is_lu_busy_func(sel, tree_name:str) -> bool:
+        try: 
+            prefix = tree_name[:7]
+            suffix = tree_name[-5:]
+            return prefix == "root.lu" and suffix == ".busy"
+        except:
+            return False
+
     def _is_property_access_func(self, func_name:str) -> bool:
-        suffix = func_name[-4:]
-        return (suffix == "_set") or (suffix == "_get")
+        try:
+            suffix = func_name[-4:]
+            return (suffix == "_set") or (suffix == "_get")
+        except:
+            return False
+
 
     def _is_table_a_studio_property(self, table) -> bool:
         """ Detect if a lua table contains a studio property definition
@@ -155,7 +180,7 @@ class CmdTreeCompiler():
         Implementation
         --------------
         This detection method is coupled with the studio lua script "make_property.lua"
-        This script implements the metatable implementation to access variables.
+        This script implements the 'meta table' implementation to access variables.
         for each variable it create a pair of set/get function.
         """
         found_access_func = False
@@ -163,7 +188,7 @@ class CmdTreeCompiler():
 
         # search for key word
         for item_name, _ in table.items():
-            if item_name == "value":
+            if item_name == "value" or item_name == "vector":
                 found_value = True  
                 break
 
@@ -181,11 +206,28 @@ class CmdTreeCompiler():
     def _is_enum_property(self, prop_elements:dict[str, str]) -> bool:
         return "enum" in prop_elements
         
+    def _is_vector_property(self, prop_elements:dict[str, str]) -> bool:
+        return "vector" in prop_elements
+        
+    def _is_power_of_two(self, n:int):
+        return (n != 0) and (n & (n-1) == 0)
+
+    def _is_enum_mask(self, enum_table) -> bool:
+        has_only_bitmask = True
+        for item_name, item_value in enum_table.items():
+            if item_name != "all" and not(item_name == "none" and item_value == 0):
+                has_only_bitmask &= self._is_power_of_two(int(item_value))
+                if has_only_bitmask == False: 
+                    break
+        return has_only_bitmask
+
     def _convert_enum_str_to_enum_class_name(self, enum_name:str) -> str:
         class_name = enum_name
         class_name = self.regex_non_enum_chars.sub('_', class_name)
         if self.regex_first_char_a_number.match(class_name):
             class_name =  "num_" + class_name
+        if keyword.iskeyword(class_name):
+            class_name =  class_name + "_"
         return class_name
 
     def _get_correct_number_type(self, lua_tree_name : str,  item_name : str) -> LuaType:
@@ -209,9 +251,18 @@ class CmdTreeCompiler():
                 property_elements[item_name] = get_lua_type_str(item_type)
         return property_elements
 
+    def _get_studio_attribute_elements(self, attribute_table) -> dict[str, str]:
+        """ Extract from attribute style lua_table a list of all attribute access functions and enum list
+            returns a dict with key:name, val:type
+        """
+        attr_elements: dict[str, str] = {}
+        for item_name, item_value in attribute_table.items():
+            attr_elements[item_name] = get_lua_type_str(item_value)
+        return attr_elements
+
     def compile_command_tree_to_file(self, studio_cmd_tree, table_version_hash: str, python_file: pathlib.Path, root_node_name:str) -> bool:
         done = False
-
+        print(f"Creating new wrapper classes to file : {python_file}")
         # create file content
         self._file_content = [""]
         self._write_cmd_tree_node(studio_cmd_tree, class_name=root_node_name.capitalize(), lua_tree_name=root_node_name)
@@ -221,7 +272,7 @@ class CmdTreeCompiler():
         try:
             f =  open(python_file, "w")    
         except:
-            self._last_error = "could not create file: '{python_file}'"
+            self._last_error = "Could not create file: '{python_file}'"
             return done
         
         try:
@@ -229,9 +280,11 @@ class CmdTreeCompiler():
                 f.write(l)
             done = True
         except:
-            self._last_error = "could not write to file: '{python_file}'"
+            self._last_error = "Could not write to file: '{python_file}'"
 
         f.close()
+
+        if not done: print(self._last_error)
         return done
 
     def create_pretty_print_command_table(self, table) -> str:
@@ -250,7 +303,7 @@ class CmdTreeCompiler():
         class_list = [] 
         class_list.append(f"# studio_wrapper.py\n")
         class_list.append(f"\n")
-        class_list.append(f"from enum import Enum\n")
+        class_list.append(f"import enum\n")
         class_list.append(f"from typing import Any\n")
         class_list.append(f"import nanosurf.lib.spm.studio.wrapper as wrap\n")
         class_list.append(f"\n")
@@ -267,32 +320,67 @@ class CmdTreeCompiler():
             class_list.append(f"class {class_name}(wrap.CmdTreeProp):\n")
             prop_elements = self._get_studio_property_elements(table, lua_tree_name)
 
-            is_prop_enum_type = self._is_enum_property(prop_elements)
-            if is_prop_enum_type:
+            is_enum_type = self._is_enum_property(prop_elements)
+            if is_enum_type:
                 enum_table = table["enum"]
+                is_enum_mask_type = self._is_enum_mask(enum_table)
                 class_list.append("\n")
-                class_list.append(f"    class EnumType(Enum):\n")
-                for _, enum_name in enum_table.items():
-                    class_list.append(f"        {self._convert_enum_str_to_enum_class_name(enum_name)} = '{enum_name}'\n")
+                if is_enum_mask_type:
+                    class_list.append(f"    class ValueMask(enum.IntEnum):\n")
+                else:
+                    class_list.append(f"    class ValueEnum(enum.Enum):\n")
+                for enum_name, enum_value in enum_table.items():
+                    class_list.append(f"        {self._convert_enum_str_to_enum_class_name(enum_name)} = {enum_value}\n")
                 class_list.append("\n")
 
             class_list.append(f"    def __init__(self, context: 'StudioScriptContext'):\n")
             class_list.append(f"        super().__init__()\n")
             class_list.append(f"        self._context = context\n")
             class_list.append(f"        self._lua_tree_name = '{lua_tree_name}'\n")
-            class_list.append(f"        self._lua_value_type = wrap.LuaType('{prop_elements['value']}')\n")
+            if self._is_vector_property(prop_elements):
+                class_list.append(f"        self._lua_value_type = wrap.LuaType('{prop_elements['vector']}')\n")
+            else:
+                class_list.append(f"        self._lua_value_type = wrap.LuaType('{prop_elements['value']}')\n")
             class_list.append("\n")
 
             for value_name, value_type in prop_elements.items():
-                if (value_name == "value") and is_prop_enum_type:
-                    value_type = "EnumType"
+                if value_name == "value": 
+                    if is_enum_type:
+                        if is_enum_mask_type:
+                            class_list.append(f"    @property\n")
+                            class_list.append(f"    def {value_name}(self) -> int:\n")
+                            class_list.append(f"        return int(self._context.get('{lua_tree_name}.value_raw'))\n")
+                            class_list.append("\n")
+                            class_list.append(f"    @{value_name}.setter\n")
+                            class_list.append(f"    def {value_name}(self, new_val:int):\n")
+                            class_list.append(f"        self._context.set('{lua_tree_name}.value_raw', int(new_val))\n")
+                            class_list.append("\n")
+                        else:                        
+                            class_list.append(f"    @property\n")
+                            class_list.append(f"    def {value_name}(self) -> ValueEnum:\n")
+                            class_list.append(f"        return {class_name}.ValueEnum(self._context.get('{lua_tree_name}.value_raw'))\n")
+                            class_list.append("\n")
+                            class_list.append(f"    @{value_name}.setter\n")
+                            class_list.append(f"    def {value_name}(self, new_val:ValueEnum):\n")
+                            class_list.append(f"        self._context.set('{lua_tree_name}.value_raw', new_val.value)\n")
+                            class_list.append("\n")
+                    else:
+                        class_list.append(f"    @property\n")
+                        class_list.append(f"    def {value_name}(self) -> {value_type}:\n")
+                        class_list.append(f"        return {value_type}(self._context.get('{lua_tree_name}.{value_name}'))\n")
+                        class_list.append("\n")
+                        class_list.append(f"    @{value_name}.setter\n")
+                        class_list.append(f"    def {value_name}(self, new_val:{value_type}):\n")
+                        class_list.append(f"        self._context.set('{lua_tree_name}.{value_name}', {value_type}(new_val))\n")
+                        class_list.append("\n")
+                elif value_name == "vector":
                     class_list.append(f"    @property\n")
-                    class_list.append(f"    def {value_name}(self) -> {value_type}:\n")
-                    class_list.append(f"        return {class_name}.{value_type}(self._context.get('{lua_tree_name}.{value_name}'))\n")
+                    class_list.append(f"    def value(self) -> {value_type}:\n")
+                    class_list.append(f"        return {value_type}(self._context.get('{lua_tree_name}.vector'))\n")
                     class_list.append("\n")
-                    class_list.append(f"    @{value_name}.setter\n")
-                    class_list.append(f"    def {value_name}(self, new_val:{value_type}):\n")
-                    class_list.append(f"        self._context.set('{lua_tree_name}.{value_name}', new_val.value)\n")
+                    class_list.append(f"    @value.setter\n")
+                    class_list.append(f"    def value(self, new_val:{value_type}):\n")
+                    class_list.append(f"        self._context.set('{lua_tree_name}.vector', {value_type}(new_val))\n")
                     class_list.append("\n")
                 else:
                     class_list.append(f"    @property\n")
@@ -317,13 +405,24 @@ class CmdTreeCompiler():
             for item_name, item_value in table.items():
                 item_type = get_lua_type(item_value)
                 if item_type == LuaType.Table:
+                    if (class_name == "Root") and (item_name == "seq") and g_skip_sequencer_definition:
+                        continue
                     class_list.append(f"        self.{item_name} = {class_name+item_name.capitalize()}(self._context)\n")
             class_list.append(f"\n")
 
             for item_name, item_value in table.items():
                 item_type = get_lua_type(item_value)
                 if item_type == LuaType.Function:
-                    if not(self._is_property_access_func(item_name)):
+                    if self._is_lu_trigger_func(lua_tree_name):
+                        class_list.append(f"    def {item_name}(self) -> None:\n")
+                        class_list.append(f"        return self._context.call('{lua_tree_name}.{item_name}')\n")
+                        class_list.append(f"\n")
+                    elif self._is_lu_busy_func(lua_tree_name):
+                        class_list.append(f"    @property\n")
+                        class_list.append(f"    def {item_name}(self) -> bool:\n")
+                        class_list.append(f"        return bool(self._context.call('{lua_tree_name}.{item_name}'))\n")
+                        class_list.append(f"\n")
+                    elif not(self._is_property_access_func(item_name)):
                         class_list.append(f"    def {item_name}(self, *args) -> Any:\n")
                         class_list.append(f"        return self._context.call('{lua_tree_name}.{item_name}', *args)\n")
                         class_list.append(f"\n")
@@ -351,4 +450,9 @@ class CmdTreeCompiler():
                 if item_type == LuaType.Table:
                     new_class_name = str(class_name+item_name.capitalize())
                     new_lua_tree_name = str(lua_tree_name+"."+item_name)
+
+                    # currently we skip sequencer definition 
+                    if new_lua_tree_name == "root.seq" and g_skip_sequencer_definition:
+                        continue
+                    
                     self._write_cmd_tree_node(item_value, new_class_name, new_lua_tree_name)
